<?php
/**
 * This file is part of workerman.
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the MIT-LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @author walkor<walkor@workerman.net>
 * @copyright walkor<walkor@workerman.net>
 * @link http://www.workerman.net/
 * @license http://www.opensource.org/licenses/mit-license.php MIT License
 */
namespace Workerman\Events;

class Select implements EventInterface
{
    /**
     * 所有的事件
     * @var array
     */
    public $_allEvents = array();
    
    /**
     * 所有信號事件
     * @var array
     */
    public $_signalEvents = array();
    
    /**
     * 監聽這些描述符的讀事件
     * @var array
     */
    protected $_readFds = array();
    
    /**
     * 監聽這些描述符的寫事件
     * @var array
     */
    protected $_writeFds = array();
    
    /**
     * 任務調度器，最大堆
     * {['data':timer_id, 'priority':run_timestamp], ..}
     * @var SplPriorityQueue
     */
    protected $_scheduler = null;
    
    /**
     * 定時任務
     * [[func, args, flag, timer_interval], ..]
     * @var array
     */
    protected $_task = array();
    
    /**
     * 定時器id
     * @var int
     */
    protected $_timerId = 1;
    
    /**
     * select超時時間，單位：微妙
     * @var int
     */
    protected $_selectTimeout = 100000000;
    
    /**
     * 構造函數
     * @return void
     */
    public function __construct()
    {
        // 創建一個管道，放入監聽讀的描述符集合中，避免空輪詢
        $this->channel = stream_socket_pair(STREAM_PF_INET, STREAM_SOCK_STREAM, STREAM_IPPROTO_IP);
        if($this->channel)
        {
            stream_set_blocking($this->channel[0], 0);
            $this->_readFds[0] = $this->channel[0];
        }
        // 初始化優先隊列(最大堆)
        $this->_scheduler = new \SplPriorityQueue();
        $this->_scheduler->setExtractFlags(\SplPriorityQueue::EXTR_BOTH);
    }
    
    /**
     * 添加事件及處理函數
     * @see Events\EventInterface::add()
     */
    public function add($fd, $flag, $func, $args = null)
    {
        switch ($flag)
        {
            case self::EV_READ:
                $fd_key = (int)$fd;
                $this->_allEvents[$fd_key][$flag] = array($func, $fd);
                $this->_readFds[$fd_key] = $fd;
                break;
            case self::EV_WRITE:
                $fd_key = (int)$fd;
                $this->_allEvents[$fd_key][$flag] = array($func, $fd);
                $this->_writeFds[$fd_key] = $fd;
                break;
            case self::EV_SIGNAL:
                throw new \Exception("not support EV_SIGNAL on Windows");
                break;
            case self::EV_TIMER:
            case self::EV_TIMER_ONCE:
                // $fd 為 定時的時間間隔，單位為秒，支持小數，能精確到0.001秒
                $run_time = microtime(true)+$fd;
                $this->_scheduler->insert($this->_timerId, -$run_time);
                $this->_task[$this->_timerId] = array($func, $args, $flag, $fd);
                $this->tick();
                return $this->_timerId++;
        }
        
        return true;
    }
    
    /**
     * 信號處理函數
     * @param int $signal
     */
    public function signalHandler($signal)
    {
        call_user_func_array($this->_signalEvents[$signal][self::EV_SIGNAL][0], array($signal));
    }
    
    /**
     * 刪除某個描述符的某類事件的監聽
     * @see Events\EventInterface::del()
     */
    public function del($fd ,$flag)
    {
        $fd_key = (int)$fd;
        switch ($flag)
        {
            case self::EV_READ:
                unset($this->_allEvents[$fd_key][$flag], $this->_readFds[$fd_key]);
                if(empty($this->_allEvents[$fd_key]))
                {
                    unset($this->_allEvents[$fd_key]);
                }
                return true;
            case self::EV_WRITE:
                unset($this->_allEvents[$fd_key][$flag], $this->_writeFds[$fd_key]);
                if(empty($this->_allEvents[$fd_key]))
                {
                    unset($this->_allEvents[$fd_key]);
                }
                return true;
            case self::EV_SIGNAL:
                break;
            case self::EV_TIMER:
            case self::EV_TIMER_ONCE;
                // $fd_key為要刪除的定時器id，即timerId
                unset($this->_task[$fd_key]);
                return true;
        }
        return false;;
    }
    
    /**
     * 檢查是否有可執行的定時任務，有的話執行
     * @return void
     */
    protected function tick()
    {
        while(!$this->_scheduler->isEmpty())
        {
            $scheduler_data = $this->_scheduler->top();
            $timer_id = $scheduler_data['data'];
            $next_run_time = -$scheduler_data['priority'];
            $time_now = microtime(true);
            if($time_now >= $next_run_time)
            {
                $this->_scheduler->extract();
                
                // 如果任務不存在，則是對應的定時器已經刪除
                if(!isset($this->_task[$timer_id]))
                {
                    continue;
                }
                
                // 任務數據[func, args, flag, timer_interval]
                $task_data = $this->_task[$timer_id];
                // 如果是持續的定時任務，再把任務加到定時隊列
                if($task_data[2] == self::EV_TIMER)
                {
                    $next_run_time = $time_now+$task_data[3];
                    $this->_scheduler->insert($timer_id, -$next_run_time);
                }
                // 嘗試執行任務
                try
                {
                    call_user_func_array($task_data[0], $task_data[1]);
                }
                catch(\Exception $e)
                {
                    echo $e;
                }
                continue;
            }
            else
            {
                // 設定超時時間
                $this->_selectTimeout = ($next_run_time - $time_now)*1000000;
                return;
            }
        }
        $this->_selectTimeout = 100000000;
    }
    
    /**
     * 刪除所有定時器
     * @return void
     */
    public function clearAllTimer()
    {
        $this->_scheduler = new \SplPriorityQueue();
        $this->_scheduler->setExtractFlags(\SplPriorityQueue::EXTR_BOTH);
        $this->_task = array();
    }
    
    /**
     * 主循環
     * @see Events\EventInterface::loop()
     */
    public function loop()
    {
        $e = null;
        while (1)
        {
            $read = $this->_readFds;
            $write = $this->_writeFds;
            // 等待可讀或者可寫事件
            stream_select($read, $write, $e, 0, $this->_selectTimeout);
            // 這些描述符可讀，執行對應描述符的讀回調函數
            if($read)
            {
                foreach($read as $fd)
                {
                    $fd_key = (int) $fd;
                    if(isset($this->_allEvents[$fd_key][self::EV_READ]))
                    {
                        call_user_func_array($this->_allEvents[$fd_key][self::EV_READ][0], array($this->_allEvents[$fd_key][self::EV_READ][1]));
                    }
                }
            }
            
            // 這些描述符可寫，執行對應描述符的寫回調函數
            if($write)
            {
                foreach($write as $fd)
                {
                    $fd_key = (int) $fd;
                    if(isset($this->_allEvents[$fd_key][self::EV_WRITE]))
                    {
                        call_user_func_array($this->_allEvents[$fd_key][self::EV_WRITE][0], array($this->_allEvents[$fd_key][self::EV_WRITE][1]));
                    }
                }
            }
            
            // 嘗試執行定時任務
            if(!$this->_scheduler->isEmpty())
            {
                $this->tick();
            }
        }
    }
}
